# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify CentOS Linux and Scientific Linux as rhel
%if "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "sl" || "%{__distinit}" == "sls"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# We need List::Util ≥ 1.29, and have to patch the code if we don't have it
%global old_list_util %(perl -MList::Util -e 'print (($List::Util::VERSION) < 1.29 ? 1 : 0);' 2>/dev/null || echo 0)

# Note: optional dependency Sub::Util is packaged with Scalar-List-Utils ≥ 1.40 (August 2014, F-21 onwards)

Name:		perl-Params-ValidationCompiler
Version:	0.30
Release:	1.%{__distinit}%{__distvers}
Summary:	Build an optimized subroutine parameter validator once, use it forever
License:	Artistic 2.0
URL:		https://metacpan.org/release/Params-ValidationCompiler
Source0:	https://cpan.metacpan.org/modules/by-module/Params/Params-ValidationCompiler-%{version}.tar.gz
Patch0:		Params-ValidationCompiler-0.30-old-List::Util.patch
BuildArch:	noarch
# Build
BuildRequires:	coreutils
BuildRequires:	findutils
BuildRequires:	make
BuildRequires:	perl-generators
BuildRequires:	perl-interpreter
BuildRequires:	perl(ExtUtils::MakeMaker)
# Module
BuildRequires:	perl(B)
BuildRequires:	perl(Carp)
BuildRequires:	perl(Eval::Closure)
BuildRequires:	perl(Exception::Class)
BuildRequires:	perl(Exporter)
BuildRequires:	perl(List::Util)
BuildRequires:	perl(overload)
BuildRequires:	perl(Class::XSAccessor)
BuildRequires:	perl(Scalar::Util)
BuildRequires:	perl(strict)
BuildRequires:	perl(warnings)
# Optional Functionality
%if 0%{?fedora} > 20 || 0%{?rhel} > 7
BuildRequires:	perl(Sub::Util) >= 1.40
Requires:	perl(Sub::Util) >= 1.40
%endif
# Test Suite
BuildRequires:	perl(File::Spec)
BuildRequires:	perl(Specio) >= 0.14
BuildRequires:	perl(Specio::Declare)
BuildRequires:	perl(Specio::Library::Builtins)
BuildRequires:	perl(Test2::Plugin::NoWarnings)
BuildRequires:	perl(Test2::Require::Module)
BuildRequires:	perl(Test2::V0)
BuildRequires:	perl(Test::More) >= 1.302015
BuildRequires:	perl(Test::Without::Module)
# Optional Tests (avoid build dependency cycles via Moose and DateTime)
BuildRequires:	perl(Const::Fast)
BuildRequires:	perl(CPAN::Meta) >= 2.120900
BuildRequires:	perl(CPAN::Meta::Prereqs)
BuildRequires:	perl(Hash::Util)
%if 0%{!?perl_bootstrap:1}
BuildRequires:	perl(Moose::Util::TypeConstraints)
BuildRequires:	perl(Types::Standard)
%endif
# Dependencies
Requires:	perl(:MODULE_COMPAT_%(eval "`perl -V:version`"; echo $version))
# Recommended
Requires:	perl(Class::XSAccessor)

%description
Create a customized, optimized, non-lobotomized, uncompromised, and thoroughly
specialized parameter checking subroutine.

%prep
%setup -q -n Params-ValidationCompiler-%{version}

# We need List::Util ≥ 1.29, and have to patch the code if we don't have it
%if %{old_list_util}
%patch0
%endif

%build
perl Makefile.PL INSTALLDIRS=vendor
make %{?_smp_mflags}

%install
make pure_install DESTDIR=%{buildroot}
find %{buildroot} -type f -name .packlist -delete
%{_fixperms} -c %{buildroot}

%check
make test

%files
%if 0%{?_licensedir:1}
%license LICENSE
%else
%doc LICENSE
%endif
%doc Changes CODE_OF_CONDUCT.md CONTRIBUTING.md eg/ README.md
%{perl_vendorlib}/Params/
%{_mandir}/man3/Params::ValidationCompiler.3*
%{_mandir}/man3/Params::ValidationCompiler::Compiler.3*
%{_mandir}/man3/Params::ValidationCompiler::Exceptions.3*

%changelog
* Wed Aug  1 2018 Paul Howarth <paul@city-fan.org> - 0.30-1
- Update to 0.30
  - Added a new option for named params, "return_object", which causes the
    validation sub to return an object with methods for each param rather than
    a hashref; this is a great way to avoid typos in hash keys (idea
    shamelessly stolen from Toby Inkster's Type::Params module)
  - Tweaked the POD formatting so that the table of contents generated by
    MetaCPAN is more useful
  - Optionally use Class::XSAccessor
- Switch upstream from search.cpan.org to metacpan.org

* Mon Feb 12 2018 Paul Howarth <paul@city-fan.org> - 0.27-1
- Update to 0.27
  - Fixed a bug with inlining Moose types: if a type's parent needed
    environment variables, those would not get closed over (GH#22)
  - Added a debug option to dump the source of the subroutine before it is
    eval'd

* Tue Nov 28 2017 Paul Howarth <paul@city-fan.org> - 0.26-1
- Update to 0.26
  - The exceptions.t test would fail if Sub::Util was not installed (GH#19)
  - Fix test failures on Windows (GH#20)

* Fri Nov 24 2017 Paul Howarth <paul@city-fan.org> - 0.25-1
- Update to 0.25
  - All exceptions now include a stack trace by default when treated as a
    string; this makes finding where validation failed a lot easier (GH#18)
  - The name for a subroutine is now used in some exception messages, even if
    Sub::Util cannot be installed
- Add patch to work around test suite breaking without Sub::Util (GH#19)

* Sun Apr  9 2017 Paul Howarth <paul@city-fan.org> - 0.24-1
- Update to 0.24
  - The source_for() exported by Params::ValidationCompiler did not work at all
    (GH#16)
- Drop support for EOL distributions prior to F-13:
  - Drop BuildRoot: and Group: tags
  - Drop explicit buildroot cleaning in %%install section
  - Drop explicit %%clean section
  - Drop bundled Test::More as we always have recent enough version now

* Mon Jan 23 2017 Paul Howarth <paul@city-fan.org> - 0.23-1
- Update to 0.23
  - Trying to create a validator for positional parameters where a required
    param came after one with a default did not throw an exception
  - A positional params validator with a slurpy type that had coercions did not
    return the coerced values; it returned the original values instead
- Update patch for building with old List::Util versions

* Sat Dec 31 2016 Paul Howarth <paul@city-fan.org> - 0.22-1
- Update to 0.22
  - Explicitly load the B module; previously, the code relied on this already
    being loaded by something else (GH#11)
  - Removed the alpha warning from the docs
- Update patch for building with old List::Util versions

* Wed Dec  7 2016 Paul Howarth <paul@city-fan.org> - 0.21-1
- Update to 0.21
  - Switched to using GitHub issues

* Tue Dec  6 2016 Paul Howarth <paul@city-fan.org> - 0.20-1
- Update to 0.20
  - The keys for parameter specifications are now validated, and if an unknown
    key is seen then an exception will be thrown; this will help you catch
    typos in your parameter specification (GH#8)
- Update patch for building with old List::Util versions

* Tue Nov 22 2016 Paul Howarth <paul@city-fan.org> - 0.19-1
- Update to 0.19
  - Non-inlinable Specio types caused a syntax error when used with positional
    params
  - Positional params with coercions and defaults did not work properly; the
    coerced value and the default would simply not be returned in any case

* Mon Nov 14 2016 Paul Howarth <paul@city-fan.org> - 0.18-1
- Update to 0.18
  - Using coercions with positional parameters could cause a "Modification of a
    read-only value attempted" exception when the generated code tried to
    assign to elements of @_; this is now fixed by making a copy if any of the
    types have a coercion
  - Using Moose types with coercions in a positional params check would cause
    invalid code to be generated; this could also happen with Type::Tiny if
    either the type or a coercion could not be inlined

* Mon Nov  7 2016 Paul Howarth <paul@city-fan.org> - 0.17-1
- Update to 0.17
  - When using positional parameters, parameters with a default are now
    optional; for named parameters, this was already the case

* Fri Nov  4 2016 Paul Howarth <paul@city-fan.org> - 0.16-1
- Update to 0.16
  - Previously, using a default with a positional parameter would result in an
    error when compiling the validator subroutine; defaults now work with
    positional parameters (GH#5)
  - Moose and Specio types (and coercions) that provide variables to close over
    when being inlined did not always compile properly; most notably, this was
    not being handled at all for Moose types, and not completely handled for
    Specio coercions

* Thu Nov  3 2016 Paul Howarth <paul@city-fan.org> - 0.14-1
- Update to 0.14
  - Added a "named_to_list" option to support returning only the parameter
    values from a named parameter validator rather than the key-value pairs
    (GH#4)
  - Errors from calls to validation_for() now use croak so as to show up at the
    call site, rather than in the internals
- Patch out the use of List::Util if we don't have List::Util ≥ 1.29

* Thu Oct 27 2016 Paul Howarth <paul@city-fan.org> - 0.13-2
- Avoid build dependency cycles via Moose and DateTime

* Sun Sep 18 2016 Paul Howarth <paul@city-fan.org> - 0.13-1
- Initial RPM version
